@@ -1,7 +1,7 @@
 //! Frontend message serialization.
 #![allow(missing_docs)]
 
-use byteorder::{WriteBytesExt, BigEndian};
+use byteorder::{WriteBytesExt, BigEndian, ByteOrder};
 use std::error::Error;
 use std::io;
 use std::marker;
@@ -16,44 +16,24 @@ pub enum Message<'a> {
         values: &'a [Option<Vec<u8>>],
         result_formats: &'a [i16],
     },
-    CancelRequest {
-        process_id: i32,
-        secret_key: i32,
-    },
-    Close {
-        variant: u8,
-        name: &'a str,
-    },
-    CopyData {
-        data: &'a [u8],
-    },
+    CancelRequest { process_id: i32, secret_key: i32 },
+    Close { variant: u8, name: &'a str },
+    CopyData { data: &'a [u8] },
     CopyDone,
-    CopyFail {
-        message: &'a str,
-    },
-    Describe {
-        variant: u8,
-        name: &'a str,
-    },
-    Execute {
-        portal: &'a str,
-        max_rows: i32,
-    },
+    CopyFail { message: &'a str },
+    Describe { variant: u8, name: &'a str },
+    Execute { portal: &'a str, max_rows: i32 },
     Parse {
         name: &'a str,
         query: &'a str,
         param_types: &'a [Oid],
     },
-    PasswordMessage {
-        password: &'a str,
-    },
-    Query {
-        query: &'a str,
-    },
+    PasswordMessage { password: &'a str },
+    Query { query: &'a str },
+    SaslInitialResponse { mechanism: &'a str, data: &'a [u8] },
+    SaslResponse { data: &'a [u8] },
     SslRequest,
-    StartupMessage {
-        parameters: &'a [(String, String)],
-    },
+    StartupMessage { parameters: &'a [(String, String)] },
     Sync,
     Terminate,
     #[doc(hidden)]
@@ -64,19 +44,23 @@ impl<'a> Message<'a> {
     #[inline]
     pub fn serialize(&self, buf: &mut Vec<u8>) -> io::Result<()> {
         match *self {
-            Message::Bind { portal, statement, formats, values, result_formats } => {
+            Message::Bind {
+                portal,
+                statement,
+                formats,
+                values,
+                result_formats,
+            } => {
                 let r = bind(portal,
                              statement,
                              formats.iter().cloned(),
                              values,
-                             |v, buf| {
-                                 match *v {
-                                     Some(ref v) => {
-                                         buf.extend_from_slice(v);
-                                         Ok(IsNull::No)
-                                     }
-                                     None => Ok(IsNull::Yes),
+                             |v, buf| match *v {
+                                 Some(ref v) => {
+                                     buf.extend_from_slice(v);
+                                     Ok(IsNull::No)
                                  }
+                                 None => Ok(IsNull::Yes),
                              },
                              result_formats.iter().cloned(),
                              buf);
@@ -86,20 +70,27 @@ impl<'a> Message<'a> {
                     Err(BindError::Serialization(e)) => Err(e),
                 }
             }
-            Message::CancelRequest { process_id, secret_key } => {
-                Ok(cancel_request(process_id, secret_key, buf))
-            }
+            Message::CancelRequest {
+                process_id,
+                secret_key,
+            } => Ok(cancel_request(process_id, secret_key, buf)),
             Message::Close { variant, name } => close(variant, name, buf),
             Message::CopyData { data } => copy_data(data, buf),
             Message::CopyDone => Ok(copy_done(buf)),
             Message::CopyFail { message } => copy_fail(message, buf),
             Message::Describe { variant, name } => describe(variant, name, buf),
             Message::Execute { portal, max_rows } => execute(portal, max_rows, buf),
-            Message::Parse { name, query, param_types } => {
-                parse(name, query, param_types.iter().cloned(), buf)
-            }
+            Message::Parse {
+                name,
+                query,
+                param_types,
+            } => parse(name, query, param_types.iter().cloned(), buf),
             Message::PasswordMessage { password } => password_message(password, buf),
             Message::Query { query: q } => query(q, buf),
+            Message::SaslInitialResponse { mechanism, data } => {
+                sasl_initial_response(mechanism, data, buf)
+            }
+            Message::SaslResponse { data } => sasl_response(data, buf),
             Message::SslRequest => Ok(ssl_request(buf)),
             Message::StartupMessage { parameters } => {
                 startup_message(parameters.iter().map(|&(ref k, ref v)| (&**k, &**v)), buf)
@@ -119,10 +110,10 @@ fn write_body<F, E>(buf: &mut Vec<u8>, f: F) -> Result<(), E>
     let base = buf.len();
     buf.extend_from_slice(&[0; 4]);
 
-    try!(f(buf));
+    f(buf)?;
 
-    let size = try!(i32::from_usize(buf.len() - base));
-    (&mut buf[base..base + 4]).write_i32::<BigEndian>(size).unwrap();
+    let size = i32::from_usize(buf.len() - base)?;
+    BigEndian::write_i32(&mut buf[base..], size);
     Ok(())
 }
 
@@ -147,28 +138,28 @@ impl From<io::Error> for BindError {
 
 #[inline]
 pub fn bind<I, J, F, T, K>(portal: &str,
-                        statement: &str,
-                        formats: I,
-                        values: J,
-                        mut serializer: F,
-                        result_formats: K,
-                        buf: &mut Vec<u8>)
-                        -> Result<(), BindError>
+                           statement: &str,
+                           formats: I,
+                           values: J,
+                           mut serializer: F,
+                           result_formats: K,
+                           buf: &mut Vec<u8>)
+                           -> Result<(), BindError>
     where I: IntoIterator<Item = i16>,
           J: IntoIterator<Item = T>,
           F: FnMut(T, &mut Vec<u8>) -> Result<IsNull, Box<Error + marker::Sync + Send>>,
-          K: IntoIterator<Item = i16>,
+          K: IntoIterator<Item = i16>
 {
     buf.push(b'B');
 
     write_body(buf, |buf| {
-        try!(buf.write_cstr(portal));
-        try!(buf.write_cstr(statement));
-        try!(write_counted(formats, |f, buf| buf.write_i16::<BigEndian>(f), buf));
-        try!(write_counted(values,
-                           |v, buf| write_nullable(|buf| serializer(v, buf), buf),
-                           buf));
-        try!(write_counted(result_formats, |f, buf| buf.write_i16::<BigEndian>(f), buf));
+        buf.write_cstr(portal)?;
+        buf.write_cstr(statement)?;
+        write_counted(formats, |f, buf| buf.write_i16::<BigEndian>(f), buf)?;
+        write_counted(values,
+                      |v, buf| write_nullable(|buf| serializer(v, buf), buf),
+                      buf)?;
+        write_counted(result_formats, |f, buf| buf.write_i16::<BigEndian>(f), buf)?;
 
         Ok(())
     })
@@ -184,11 +175,11 @@ fn write_counted<I, T, F, E>(items: I, mut serializer: F, buf: &mut Vec<u8>) ->
     buf.extend_from_slice(&[0; 2]);
     let mut count = 0;
     for item in items {
-        try!(serializer(item, buf));
+        serializer(item, buf)?;
         count += 1;
     }
-    let count = try!(i16::from_usize(count));
-    (&mut buf[base..base + 2]).write_i16::<BigEndian>(count).unwrap();
+    let count = i16::from_usize(count)?;
+    BigEndian::write_i16(&mut buf[base..], count);
 
     Ok(())
 }
@@ -199,7 +190,8 @@ pub fn cancel_request(process_id: i32, secret_key: i32, buf: &mut Vec<u8>) {
         buf.write_i32::<BigEndian>(80877102).unwrap();
         buf.write_i32::<BigEndian>(process_id).unwrap();
         buf.write_i32::<BigEndian>(secret_key)
-    }).unwrap();
+    })
+            .unwrap();
 }
 
 #[inline]
@@ -246,7 +238,7 @@ pub fn describe(variant: u8, name: &str, buf: &mut Vec<u8>) -> io::Result<()> {
 pub fn execute(portal: &str, max_rows: i32, buf: &mut Vec<u8>) -> io::Result<()> {
     buf.push(b'E');
     write_body(buf, |buf| {
-        try!(buf.write_cstr(portal));
+        buf.write_cstr(portal)?;
         buf.write_i32::<BigEndian>(max_rows).unwrap();
         Ok(())
     })
@@ -258,9 +250,9 @@ pub fn parse<I>(name: &str, query: &str, param_types: I, buf: &mut Vec<u8>) -> i
 {
     buf.push(b'P');
     write_body(buf, |buf| {
-        try!(buf.write_cstr(name));
-        try!(buf.write_cstr(query));
-        try!(write_counted(param_types, |t, buf| buf.write_u32::<BigEndian>(t), buf));
+        buf.write_cstr(name)?;
+        buf.write_cstr(query)?;
+        write_counted(param_types, |t, buf| buf.write_u32::<BigEndian>(t), buf)?;
         Ok(())
     })
 }
@@ -277,6 +269,24 @@ pub fn query(query: &str, buf: &mut Vec<u8>) -> io::Result<()> {
     write_body(buf, |buf| buf.write_cstr(query))
 }
 
+#[inline]
+pub fn sasl_initial_response(mechanism: &str, data: &[u8], buf: &mut Vec<u8>) -> io::Result<()> {
+    buf.push(b'p');
+    write_body(buf, |buf| {
+        buf.write_cstr(mechanism)?;
+        let len = i32::from_usize(data.len())?;
+        buf.write_i32::<BigEndian>(len)?;
+        buf.extend_from_slice(data);
+        Ok(())
+    })
+}
+
+#[inline]
+pub fn sasl_response(data: &[u8], buf: &mut Vec<u8>) -> io::Result<()> {
+    buf.push(b'p');
+    write_body(buf, |buf| Ok(buf.extend_from_slice(data)))
+}
+
 #[inline]
 pub fn ssl_request(buf: &mut Vec<u8>) {
     write_body(buf, |buf| buf.write_i32::<BigEndian>(80877103)).unwrap();
@@ -289,8 +299,8 @@ pub fn startup_message<'a, I>(parameters: I, buf: &mut Vec<u8>) -> io::Result<()
     write_body(buf, |buf| {
         buf.write_i32::<BigEndian>(196608).unwrap();
         for (key, value) in parameters {
-            try!(buf.write_cstr(key.as_ref()));
-            try!(buf.write_cstr(value.as_ref()));
+            buf.write_cstr(key.as_ref())?;
+            buf.write_cstr(value.as_ref())?;
         }
         buf.push(0);
         Ok(())
